
# Example output from 'sudoku -g -fcompact'

# % randomly generated - medium
# 2....961.
# .6..2.35.
# 3..6....9
# ....8..36
# ..3...8..
# 92..7....
# 8....1..5
# .47.6..9.
# .964....7

proc setblock {col row value} {
   global grid presets

   if {$value == "."} { set grid($row$col) "" ; return }
   set grid($row$col) $value
   .grid.$row$col configure -font {Times 16 bold} -foreground #d90000
   lappend presets [list $row$col $value]
}

proc setgrid {idx rlist} {
   setblock $idx a [lindex $rlist 0]
   setblock $idx b [lindex $rlist 1]
   setblock $idx c [lindex $rlist 2]
   setblock $idx d [lindex $rlist 3]
   setblock $idx e [lindex $rlist 4]
   setblock $idx f [lindex $rlist 5]
   setblock $idx g [lindex $rlist 6]
   setblock $idx h [lindex $rlist 7]
   setblock $idx i [lindex $rlist 8]
}


proc gen_valid_board {} {
   global oklevels sudoku101
   set retlist [list]
   set skip 0

   set validboard 0
   while {!$skip} {
      set bd [exec $sudoku101 -g25 -fcompact -t win32/template]
      set flist [split $bd \n]
      foreach line $flist {
         if {$skip} { continue }
         if {[regexp {^% randomly generated - (.*)$} $line a rating]} {
            if {$validboard} {
               set skip 1
            } elseif {[lsearch -exact $oklevels $rating] >= 0} {
               set validboard 1
               lappend retlist $line
            }
         } elseif {$validboard} {
            lappend retlist $line
         }
      }
   }
   return $retlist
}


proc getboard {{immed 1}} {
   global presets sudoku101 oklevels ingame
   set presets [list]

   if {!$immed} {
      set ans [tk_messageBox -type yesno -default yes \
                              -message "Get a new board?  \[Y/n]  "]
      if {$ans == "no"} { return }
   }

   clearall
   wm title . "Getting new board..."
   set blist [gen_valid_board]
   foreach {stat ra rb rc rd re rf rg rh ri} $blist { break }
   set level [lindex $stat end]
   wm title . "Random gen - $level"

   set rlist [split $ra ""] ; setgrid a $rlist
   set rlist [split $rb ""] ; setgrid b $rlist
   set rlist [split $rc ""] ; setgrid c $rlist
   set rlist [split $rd ""] ; setgrid d $rlist
   set rlist [split $re ""] ; setgrid e $rlist
   set rlist [split $rf ""] ; setgrid f $rlist
   set rlist [split $rg ""] ; setgrid g $rlist
   set rlist [split $rh ""] ; setgrid h $rlist
   set rlist [split $ri ""] ; setgrid i $rlist
   set ingame 0
}

# This next bit actually interfaces with sudoku_solver, not sudoku101.exe.
# 

# note:  grid coordinates are:  grid(xy), x (left->right a->i) and
# y (top->bottom a->i)


proc solve {{mode display}} {
   global grid allow_cheats sudoku_solver
   set s ""
   foreach y [list a b c d e f g h i] {
      foreach x [list a b c d e f g h i] {
         set c [.grid.$x$y get]
         if {$c == ""} { set c . }
         set s $s$c
      }
   }
   set f [open .solvetmp w]
   puts -nonewline $f $s
   close $f
   set result [lindex [split [exec $sudoku_solver -aG -f .solvetmp] \n] 0]
   set reslist  [split [lindex [split $result \n] 0] ""]

   set i 0
   foreach y [list a b c d e f g h i] {
      foreach x [list a b c d e f g h i] {
         set idx($x$y) $i
         incr i
      }
   }

   if {[streq $mode "square"]} {
      if {$allow_cheats == 0} { return }
      set sq [focus]
      if {[regexp {.grid.([a-i][a-i])} $sq a sqidx]} {
         set c [lindex $reslist $idx($sqidx)]
         set grid($sqidx) $c
         update
      }
      return
   } else {
      foreach y [list a b c d e f g h i] {
         foreach x [list a b c d e f g h i] {
            set c [lindex $reslist $idx($x$y)]
            set grid($x$y) $c
         }
      }
   }
}


